/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DistributedJMXServerManager.java 16:34:49 ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.communication.jmx;

import javax.management.MBeanServer;

import org.objectweb.petals.PetalsException;

/**
 * This class is a manager of JMX server. It is used to get a connection to a
 * JMX Server on a distant container.
 * 
 * @author ddesjardins - eBMWebsourcing
 */
public interface JMXService {

    /**
     * Create a distributed JMX server, kind of JMX proxy to a specific
     * container
     * 
     * @param containerName
     *            qualified name of the container
     * @return DistributedJMXServer
     * @throws PetalsException
     */
    DistributedJMXServer createDistributedJMXServer(String containerName)
        throws PetalsException;

    /**
     * Check if a container is started
     * 
     * @param containerName
     *            name of the container; must be non null
     * @return true if the container is started; false is the container is
     *         stopped or if a problem occurs
     */
    boolean isContainerStarted(String containerName);
    
    /** 
     *  Get the local jmx server
     * 
     * @return the locel jmx server
     */
    public MBeanServer getLocalJMXServer();

}
