/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: DeploymentServiceImpl.java 154 2006-03-27 15:30:10Z ofabre $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.management.deployment;

import java.net.URI;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Set;

import javax.jbi.management.LifeCycleMBean;

import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.Requires;
import org.objectweb.petals.PetalsException;
import org.objectweb.petals.container.ContainerService;
import org.objectweb.petals.container.lifecycle.ComponentLifeCycle;
import org.objectweb.petals.container.lifecycle.LifeCycleAbstract;
import org.objectweb.petals.container.lifecycle.ServiceAssemblyLifeCycle;
import org.objectweb.petals.container.lifecycle.ServiceUnitLifeCycle;
import org.ow2.petals.jbi.descriptor.JBIDescriptorBuilder;
import org.ow2.petals.jbi.descriptor.JBIDescriptorConstants;
import org.ow2.petals.jbi.descriptor.JBIDescriptorException;
import org.ow2.petals.jbi.descriptor.ServiceUnit;
import org.objectweb.petals.jbi.management.admin.AdminServiceMBean;
import org.objectweb.petals.jbi.management.task.TaskContextConstants;
import org.objectweb.petals.jbi.management.task.deployment.DeploymentContextConstants;
import org.objectweb.petals.jbi.management.task.deployment.DeploymentTaskFactory;
import org.objectweb.petals.jbi.management.util.PackageHandler;
import org.objectweb.petals.jbi.management.util.XMLResult;
import org.objectweb.petals.jbi.management.util.XMLResult.CauseFramework;
import org.objectweb.petals.jbi.management.util.XMLResult.MessageType;
import org.objectweb.petals.jbi.management.util.XMLResult.TaskResult;
import org.objectweb.petals.jbi.messaging.registry.EndpointService;
import org.objectweb.petals.kernel.server.FractalHelper;
import org.objectweb.petals.platform.repository.RepositoryService;
import org.objectweb.petals.platform.systemstate.ServiceAssemblyState;
import org.objectweb.petals.platform.systemstate.SystemStateService;
import org.objectweb.petals.processor.TaskProcessor;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.ParameterCheckHelper;
import org.objectweb.util.monolog.api.Logger;

/**
 * JBI deployment service implementation.
 *
 * @author Olivier Fabre - EBM Websourcing
 * @author ddesjardins - eBMWebsourcing
 * @author wjoseph - eBMWebsourcing
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = org.objectweb.petals.jbi.management.deployment.DeploymentServiceMBean.class))
public class DeploymentServiceImpl implements DeploymentServiceMBean {

	@Requires(name = "endpoint", signature = org.objectweb.petals.jbi.messaging.registry.EndpointService.class)
	private EndpointService endpointService;

	/**
	 * logger wrapper
	 */
	private LoggingUtil log;

	/**
	 * Monolog Logger instance.
	 */
	@Monolog(name = "logger")
	private Logger logger;

	/**
	 * JBI Container :: JMX Agent Admin
	 */
	@Requires(name = "admin", signature = org.objectweb.petals.jbi.management.admin.AdminServiceMBean.class)
	private AdminServiceMBean adminService;

	/**
	 * Package Handler
	 */
	private PackageHandler packageHandler;

	/**
	 * JBI Container :: Recovery Service
	 */
	@Requires(name = "systemstate", signature = org.objectweb.petals.platform.systemstate.SystemStateService.class)
	private SystemStateService systemStateService;

	/**
	 * Platform Component :: Repository Service
	 */
	@Requires(name = "repository", signature = org.objectweb.petals.platform.repository.RepositoryService.class)
	private RepositoryService repositoryService;

	/**
	 * Container :: Container Service
	 */
	@Requires(name = "container", signature = org.objectweb.petals.container.ContainerService.class)
	protected ContainerService container;

	private DeploymentTaskFactory deploymentTaskFactory;

	/**
	 * Default constructor.
	 */
	public DeploymentServiceImpl() {
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#canDeployToComponent(java.lang.String)
	 */
	public boolean canDeployToComponent(String componentName) {
		ParameterCheckHelper.isNullParameterWithLog(componentName,
				"componentName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(componentName,
				"componentName", log);
		boolean result = false;
		ComponentLifeCycle clc = adminService
				.getComponentLifeCycleByName(componentName);
		result = clc != null
				&& LifeCycleAbstract.STARTED.equals(clc.getCurrentState())
				&& clc.getComponent().getServiceUnitManager() != null;
		return result;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#deploy(java.lang.String)
	 */
	public String deploy(String serviceAssemblyZipUrl) throws Exception {
		log.start();
		String result = null;

		/*
		 * Checking parameters
		 */
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyZipUrl,
				"serviceAssemblyZipUrl", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyZipUrl,
				"serviceAssemblyZipUrl", log);

		/*
		 * Transform URL into URI
		 */
		URI archiveURI = packageHandler.processAndGetPackageURI(
				serviceAssemblyZipUrl, false);

		try {

			/*
			 * Create deployment execution context
			 */
			HashMap<String, Object> deploymentContext = createDeploymentContext(archiveURI);

			/*
			 * Create deployment processor and fill with tasks
			 */
			TaskProcessor processor = createDeploymentProcessor(
					deploymentContext, archiveURI);

			/*
			 * Launch the process
			 */
			if (!processor.process()) {
				String msg = "An error occured during the deployment "
						+ "of the following service assembly: "
						+ serviceAssemblyZipUrl;
				log.error(msg);
				throw new Exception(msg);
			}

			/*
			 * Create the string result
			 */
			result = ((XMLResult) deploymentContext
					.get(DeploymentContextConstants.XML_RESULT)).getString();

			log.info("Service Assembly succesfully deployed: "
					+ (String) deploymentContext
							.get(TaskContextConstants.ENTITY_NAME));

		} catch (Throwable e) {
			// Catch all throwables to prevent exotic exception type (not easily
			// deserializable) throwing
			throw new Exception(e.getMessage(), e);
		}

		// log.info(result);
		log.end();
		return result;
	}

	/**
	 * Create the deployment processor and add all required task for the
	 * complete deployment of a service assembly
	 *
	 * @param deploymentContext
	 *            the deployment context
	 * @param archiveURI
	 *            the sl archive URI
	 * @return the filled deployment processor
	 */
	protected TaskProcessor createDeploymentProcessor(
			HashMap<String, Object> deploymentContext, URI archiveURI) {
		TaskProcessor processor = new TaskProcessor(deploymentContext, log);

		String scheme = archiveURI.getScheme();
		if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("ftp")) {
			processor.addTask(deploymentTaskFactory.getDownloadTask());
			processor.addTask(deploymentTaskFactory.getExtractTask());
		} else if (scheme.equalsIgnoreCase("file")) {
			if (!packageHandler.isDirectory(archiveURI)) {
				processor.addTask(deploymentTaskFactory.getExtractTask());
			} else {
				processor.addTask(deploymentTaskFactory
						.getCopyArchiveToWorkDirTask());
			}
		} else {
			throw new IllegalArgumentException("URL protocol not supported:"
					+ scheme);
		}
		processor.addTask(deploymentTaskFactory.getSaCheckPackageTask());
		processor.addTask(deploymentTaskFactory
				.getCopySAPackageToRepositoryTask());
		processor.addTask(deploymentTaskFactory
				.getCreateXMLDeploymentResultTask());
		if (scheme.equalsIgnoreCase("http") || scheme.equalsIgnoreCase("ftp")) {
			processor.addTask(deploymentTaskFactory.getExtractSUsTask());
		} else if (scheme.equalsIgnoreCase("file")) {
			if (!packageHandler.isDirectory(archiveURI)) {
				processor.addTask(deploymentTaskFactory.getExtractSUsTask());
			} else {
				processor.addTask(deploymentTaskFactory.getRenameSUsTask());
			}
		}
		processor.addTask(deploymentTaskFactory.getRegisterSALifeCycleTask());
		processor.addTask(deploymentTaskFactory.getRegisterAllConnectionTask());
		processor.addTask(deploymentTaskFactory.getDeployAllSUTask());
		processor.addTask(deploymentTaskFactory.getCreateSAStateHolderTask());
		if (scheme.equalsIgnoreCase("file")
				&& !packageHandler.isDirectory(archiveURI)) {
			processor.addTask(deploymentTaskFactory
					.getCopyArchiveToInstalledDirTask());
		}
		processor.addTask(deploymentTaskFactory.getSuccessTask());

		return processor;
	}

	/**
	 * Create the deployment context
	 *
	 * @param archiveURI
	 *            URI of the Service Assembly installation package file; must be
	 *            non-null, non-empty, and a legal URI.
	 * @return the Deployment Context HashMap
	 */
	protected HashMap<String, Object> createDeploymentContext(URI archiveURI) {
		HashMap<String, Object> deploymentContext = new HashMap<String, Object>();
		deploymentContext.put(TaskContextConstants.ARCHIVE_URI, archiveURI);
		return deploymentContext;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#getComponentsForDeployedServiceAssembly(java.lang.String)
	 */
	public String[] getComponentsForDeployedServiceAssembly(
			String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		List<String> result = new ArrayList<String>();
		ServiceAssemblyLifeCycle saLifeCycle = adminService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			List<ServiceUnit> sus = saLifeCycle.getServiceAssembly()
					.getServiceUnits();
			for (ServiceUnit unit : sus) {
				result.add(unit.getTargetComponentName());
			}
		} else {
			String msg = "Cannot retrieve service assembly with the specified name : "
					+ serviceAssemblyName;
			log.error(msg);
			throw new Exception(msg);
		}
		return result.toArray(new String[0]);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssemblies()
	 */
	public String[] getDeployedServiceAssemblies() throws Exception {
		Set<String> sas = adminService.getServiceAssemblyLifeCycles().keySet();
		return sas.toArray(new String[0]);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceAssembliesForComponent(java.lang.String)
	 */
	public String[] getDeployedServiceAssembliesForComponent(
			String componentName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(componentName,
				"componentName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(componentName,
				"componentName", log);
		Set<String> result = new HashSet<String>();
		Map<String, ServiceAssemblyLifeCycle> saLifeCycles = adminService
				.getServiceAssemblyLifeCycles();
		for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
			List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
			for (ServiceUnit unit : sus) {
				if (componentName.equals(unit.getTargetComponentName())) {
					result.add(salc.getServiceAssembly().getIdentification()
							.getName());
				}
			}
		}
		return result.toArray(new String[0]);
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#getDeployedServiceUnitList(java.lang.String)
	 */
	public String[] getDeployedServiceUnitList(String componentName)
			throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(componentName,
				"componentName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(componentName,
				"componentName", log);
		Set<String> result = new HashSet<String>();
		Map<String, ServiceAssemblyLifeCycle> saLifeCycles = adminService
				.getServiceAssemblyLifeCycles();
		for (ServiceAssemblyLifeCycle salc : saLifeCycles.values()) {
			List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
			for (ServiceUnit unit : sus) {
				if (componentName.equals(unit.getTargetComponentName())) {
					result.add(unit.getIdentification().getName());
				}
			}
		}
		return result.toArray(new String[0]);
	}

	public String getServiceAssemblyDescriptor(String serviceAssemblyName)
			throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		String result = "Service Assembly doesn't exist :"
				+ serviceAssemblyName;
		ServiceAssemblyLifeCycle saLifeCycle = adminService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			URI installationRoot = saLifeCycle.getInstallationRoot();
			// Load JBI descriptor as string
			URI jbiXmlUri = installationRoot
					.resolve(JBIDescriptorConstants.JBI_DESCRIPTOR_RESOURCE
							.replace('\\', '/'));
			result = getJbiDescriptorAsString(jbiXmlUri);
		}
		return result;
	}

	/**
	 *
	 * @param saName
	 * @return
	 * @throws Exception
	 */
	public String[] getServiceUnitForServiceAssembly(String saName) throws Exception {

        ParameterCheckHelper.isNullParameterWithLog(saName, "saName", log);
        Set<String> result;
        Map<String, ServiceAssemblyLifeCycle> saLifeCycles = adminService
                .getServiceAssemblyLifeCycles();

        ServiceAssemblyLifeCycle saLifeCycle = saLifeCycles.get(saName);

        if (saLifeCycle != null) {
            Map<String, ServiceUnitLifeCycle> suLifeCycles = saLifeCycle
                    .getServiceUnitsLifeCycles();
            result = suLifeCycles.keySet();

        } else {
            String msg = "Cannot retrieve service assembly with the specified name : " + saName;
            log.error(msg);
            throw new Exception(msg);
        }

        return result.toArray(new String[0]);
    }

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#getState(java.lang.String)
	 */
	public String getState(String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		String result = null;
		String msg;
		ServiceAssemblyLifeCycle saLifeCycle = adminService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			result = saLifeCycle.getCurrentState();
		} else {
			msg = "Cannot retrieve service "
					+ "assembly with the specified name : "
					+ serviceAssemblyName;
			log.error(msg);
			throw new Exception(msg);
		}
		if (!(LifeCycleMBean.SHUTDOWN.equals(result)
				|| LifeCycleMBean.STARTED.equals(result) || LifeCycleMBean.STOPPED
				.equals(result))) {
			msg = "Service Assembly state isn't correctly defined";
			log.error(msg);
			throw new Exception(msg);
		}
		return result;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#shutDown(java.lang.String)
	 */
	public String shutDown(String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		log.start();

		String msg = null;
		XMLResult xmlResult = new XMLResult("shutdown");

		ServiceAssemblyLifeCycle saLifeCycle = adminService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			try {
				saLifeCycle.shutDown();
			} catch (Throwable e) {
				msg = e.getMessage();
				log.error(msg, e);
				throw new Exception(msg);
			}
		} else {
			msg = "The service assembly \"" + serviceAssemblyName
					+ "\" is not deployed";
			throw new Exception(msg);
		}
		xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
				"Successfully shutdown service assembly : {1}",
				new String[] { serviceAssemblyName }, CauseFramework.YES);
		String output = xmlResult.getString();
		// try {
		// XMLResult.validateManagementMessage(output, this.getClass()
		// .getClassLoader().getResourceAsStream(
		// "schema/management-message.xsd"));
		// } catch (Exception e1) {
		// log.error("Problem while validating management message", e1);
		// }
		log.info(serviceAssemblyName + " successfully shutdown");
		log.end();
		return output;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#start(java.lang.String)
	 */
	public String start(String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		log.start();

		String msg = null;
		XMLResult xmlResult = new XMLResult("start");

		ServiceAssemblyLifeCycle saLifeCycle = adminService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			try {
				// saLifeCycle.start();
				FractalHelper.getFractalHelper().startComponent(
						saLifeCycle.getComponent());
			} catch (Exception e) {
				e.printStackTrace();
				msg = e.getMessage();
				throw new Exception(msg);
			}
		} else {
			msg = "The service assembly \"" + serviceAssemblyName
					+ "\" is not deployed";
			throw new Exception(msg);
		}
		xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
				"Successfully started service assembly : {1}",
				new String[] { serviceAssemblyName }, CauseFramework.YES);
		String output = xmlResult.getString();
		// try {
		// XMLResult.validateManagementMessage(output, this.getClass()
		// .getClassLoader().getResourceAsStream(
		// "schema/management-message.xsd"));
		// } catch (Exception e1) {
		// log.error("Problem while validating management message", e1);
		// }

		log.info(serviceAssemblyName + " successfully started");
		log.end();
		return output;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#stop(java.lang.String)
	 */
	public String stop(String serviceAssemblyName) throws Exception {
		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		log.start();

		String msg = null;
		XMLResult xmlResult = new XMLResult("stop");

		ServiceAssemblyLifeCycle saLifeCycle = adminService
				.getServiceAssemblyByName(serviceAssemblyName);
		if (saLifeCycle != null) {
			try {
				saLifeCycle.stop();
				FractalHelper.getFractalHelper().stopComponent(
						saLifeCycle.getComponent());
			} catch (Throwable e) {
				msg = e.getMessage();
				log.error(msg, e);
				throw new Exception(msg);
			}
		} else {
			msg = "The service assembly \"" + serviceAssemblyName
					+ "\" is not deployed";
			throw new Exception(msg);
		}
		xmlResult.addFrameworkTaskResult(TaskResult.SUCCESS, MessageType.INFO,
				"Successfully stopped service assembly : {1}",
				new String[] { serviceAssemblyName }, CauseFramework.YES);
		String output = xmlResult.getString();
		// try {
		// XMLResult.validateManagementMessage(output, this.getClass()
		// .getClassLoader().getResourceAsStream(
		// "schema/management-message.xsd"));
		// } catch (Exception e1) {
		// log.error("Problem while validating management message", e1);
		// }

		log.info(serviceAssemblyName + " successfully stoped");
		log.end();
		return output;
	}

	/*
	 * (non-Javadoc)
	 *
	 * @see javax.jbi.management.DeploymentServiceMBean#undeploy(java.lang.String)
	 */
	public String undeploy(String serviceAssemblyName) throws Exception {
		log.start();

		String output = "";

		ParameterCheckHelper.isNullParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceAssemblyName,
				"serviceAssemblyName", log);
		try {

			/*
			 * Create undeployment execution context
			 */
			HashMap<String, Object> undeploymentContext = createUndeploymentContext(serviceAssemblyName);

			/*
			 * Create undeployment processor and fill with tasks
			 */
			TaskProcessor processor = createUndeploymentProcessor(undeploymentContext);

			/*
			 * Launch the process
			 */
			if (!processor.process()) {
				String msg = "An error occured during the undeployment "
						+ "of the following service assembly: "
						+ serviceAssemblyName;
				throw new Exception(msg);
			}

			/*
			 * Create the string result
			 */
			output = ((XMLResult) undeploymentContext
					.get(DeploymentContextConstants.XML_RESULT)).getString();

		} catch (Throwable e) {
			// Catch all throwables to prevent exotic exception type (not easily
			// deserializable) throwing
			throw new Exception(e);
		}

		log.info(serviceAssemblyName + " successfully undeployed");
		log.end();
		return output;
	}

	/**
	 * Create the undeployment processor and add all required task for the
	 * complete undeployment of a service assembly
	 *
	 * @param undeploymentContext
	 *            the deployment context
	 * @return filled undeployment processor
	 */
	protected TaskProcessor createUndeploymentProcessor(
			HashMap<String, Object> undeploymentContext) {
		TaskProcessor processor = new TaskProcessor(undeploymentContext, log);

		processor.addTask(deploymentTaskFactory
				.getCreateXMLUndeploymentResultTask());
		processor.addTask(deploymentTaskFactory.getRetrieveSALifeCycleTask());
		processor.addTask(deploymentTaskFactory.getUndeployAllSUTask());
		processor.addTask(deploymentTaskFactory.getRemoveAllConnectionsTask());
		processor.addTask(deploymentTaskFactory.getUnregisterSALifeCycleTask());
		processor.addTask(deploymentTaskFactory.getRemoveSAStateHolderTask());
		processor.addTask(deploymentTaskFactory.getRemoveSAFolderTask());
		processor.addTask(deploymentTaskFactory
				.getDeleteArchiveFromInstalledDirTask());

		return processor;
	}

	/**
	 * Create the undeployment context
	 *
	 * @param serviceAssemblyName
	 *            the service assembly name
	 * @return the undeployment context
	 * @throws PetalsException
	 *             if error occured during JBI descriptor loading
	 */
	protected HashMap<String, Object> createUndeploymentContext(
			String serviceAssemblyName) throws PetalsException {
		HashMap<String, Object> undeploymentContext = new HashMap<String, Object>();

		ServiceAssemblyState state = systemStateService
				.getServiceAssemblyState(serviceAssemblyName);

		undeploymentContext.put(TaskContextConstants.ENTITY_NAME,
				serviceAssemblyName);
		undeploymentContext.put(TaskContextConstants.ARCHIVE_URI,
				packageHandler.processAndGetPackageURI(state.getArchiveURL(),
						false));
		return undeploymentContext;
	}

	/**
	 * Return a string representation of the JBI descriptor. This descriptor in
	 * found thanks to the jbiXmlUri
	 *
	 * @param jbiXmlUri
	 *            the URI of the deployment descriptor
	 * @return a string representing the JBI deployment descriptor
	 * @throws JBIDescriptorException
	 */
	protected String getJbiDescriptorAsString(URI jbiXmlUri)
			throws JBIDescriptorException {
		return JBIDescriptorBuilder.getInstance().getJbiDescriptorAsString(
				jbiXmlUri);
	}

	@LifeCycle(on = LifeCycleType.START)
	protected void start() {
		log = new LoggingUtil(logger);
		log.start();

		/*
		 * Initialize package handler
		 */
		packageHandler = new PackageHandler(logger, repositoryService);

		/*
		 * Initialize DeploymentTaskFactory
		 */
		deploymentTaskFactory = new DeploymentTaskFactory(packageHandler,
				endpointService, this, systemStateService, repositoryService,
				log, adminService, container);

		log.end();
	}

	@LifeCycle(on = LifeCycleType.STOP)
	public void stop() {
		log.start();

		logger = null;

		log.end();
	}

	/**
	 * Shutdown the DeploymentService. All service assemblies are undeployed.
	 */
	public void shutdown() throws Exception {
		// Undeploy all service assemblies
		undeployAllServiceAssemblies(true);
	}

	public String[] startAllServiceAssemblies() {

		List<String> startedSAs = new ArrayList<String>();

		/*
		 * Try to start all service assemblies, if a SA can't be started, it's
		 * skipped and the starting process ins't aborted.
		 */
		for (String name : systemStateService
				.getAllDeployedServiceAssemblyNames()) {
			try {
				start(name);
				startedSAs.add(name);
			} catch (Throwable e) {
				log.warning("The following SA can't be started: " + name
						+ ". Cause: " + e.getMessage(), e);
			}
		}

		return startedSAs.toArray(new String[startedSAs.size()]);
	}

	public String[] stopAllServiceAssemblies() {

		List<String> stopedSAs = new ArrayList<String>();

		/*
		 * Try to stop all service assemblies. If a SA can't be stoped, it's
		 * skipped and the stoping process ins't aborted.
		 */
		for (String name : systemStateService
				.getAllDeployedServiceAssemblyNames()) {
			try {
				stop(name);
				stopedSAs.add(name);
			} catch (Throwable e) {
				log.warning("The following SA can't be stoped: " + name
						+ ". Cause: " + e.getMessage(), e);
			}
		}

		return stopedSAs.toArray(new String[stopedSAs.size()]);
	}

	public String[] shutdownAllServiceAssemblies() {

		List<String> shutdownSAs = new ArrayList<String>();

		/*
		 * Try to shutdown all service assemblies. If a SA can't be shutdown,
		 * it's skipped and the shutdowning process ins't aborted.
		 */
		for (String name : systemStateService
				.getAllDeployedServiceAssemblyNames()) {
			try {
				shutDown(name);
				shutdownSAs.add(name);
			} catch (Throwable e) {
				log.warning("The following SA can't be shutdown: " + name
						+ ". Cause: " + e.getMessage(), e);
			}
		}

		return shutdownSAs.toArray(new String[shutdownSAs.size()]);
	}

	public String[] undeployAllServiceAssemblies(boolean force) {
		List<String> undeployedSAs = new ArrayList<String>();

		/*
		 * Try to undeploy all service assemblies. If a SA can't be undeployed,
		 * it's skipped and the undeploying process ins't aborted.
		 */
		for (String name : systemStateService
				.getAllDeployedServiceAssemblyNames()) {
			if (force) {
				if (forceUndeployServiceAssembly(name)) {
					undeployedSAs.add(name);
				}
			} else {
				try {
					undeploy(name);
					undeployedSAs.add(name);
				} catch (Throwable e) {
					log.warning("The following SA can't be undeployed: " + name
							+ ". Cause: " + e.getMessage(), e);
				}
			}
		}
		return undeployedSAs.toArray(new String[undeployedSAs.size()]);
	}

	public boolean forceUndeployServiceAssembly(String serviceAssemblyName) {
		boolean removed = false;

		String saState = LifeCycleMBean.UNKNOWN;
		try {
			saState = getState(serviceAssemblyName);
		}
		catch (Exception e) {
		    log.warning("An error occured during service assembly getState", e);
            // The SA could be in state "Unknown"
        }
		
		try {
	        if (LifeCycleMBean.STARTED.equals(saState)) {
				stop(serviceAssemblyName);
				shutDown(serviceAssemblyName);
			} else if (LifeCycleMBean.STOPPED.equals(saState)) {
				shutDown(serviceAssemblyName);
			}
		} catch (Exception e) {
			log.warning("An error occured during service assembly shutdown", e);
		}
		
		try {
	        // Here the SA should be in state "Unknown" or "Shutdown"
			undeploy(serviceAssemblyName);
			removed = true;
		} catch (Exception e) {
			log.warning("A service assembly can't be correctly force-uninstalled: "
					+ serviceAssemblyName);
		}
		return removed;
	}

	public boolean isDeployedServiceUnit(String componentName,
			String serviceUnitName) {
		ParameterCheckHelper.isNullParameterWithLog(serviceUnitName,
				"serviceUnitName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(serviceUnitName,
				"serviceUnitName", log);
		ParameterCheckHelper.isNullParameterWithLog(componentName,
				"componentName", log);
		ParameterCheckHelper.isEmptyParameterWithLog(componentName,
				"componentName", log);
		boolean result = false;
		Map<String, ServiceAssemblyLifeCycle> saLifeCycles = adminService
				.getServiceAssemblyLifeCycles();
		for (Iterator iter = saLifeCycles.values().iterator(); iter.hasNext()
				&& !result;) {
			ServiceAssemblyLifeCycle salc = (ServiceAssemblyLifeCycle) iter
					.next();
			List<ServiceUnit> sus = salc.getServiceAssembly().getServiceUnits();
			for (Iterator iter2 = sus.iterator(); iter2.hasNext() && !result;) {
				ServiceUnit unit = (ServiceUnit) iter2.next();
				if (componentName.equals(unit.getTargetComponentName())
						&& serviceUnitName.equals(unit.getIdentification()
								.getName())) {
					result = true;
				}
			}
		}
		return result;
	}

}
