/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2005 Fossil E-Commerce, http://www.fossilec.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id: LinkedEndpoint.java 438 2006-05-19 11:12:26Z ddesjardins $
 * -------------------------------------------------------------------------
 */
package org.objectweb.petals.jbi.messaging.servicedesc;

import java.io.Serializable;

import javax.xml.namespace.QName;

import org.objectweb.petals.jbi.messaging.registry.EndpointService;

/**
 * This endpoint represents a "connexion". A connexion is defined in a service
 * assembly, and is like an alias for a real endpoint.
 * 
 * @version $Rev: 438 $ $Date: 2006-05-19 13:12:26 +0200 (ven., 19 mai 2006) $
 * @since Petals 1.0
 * @author alouis
 */
public class LinkedEndpoint extends AbstractEndpoint implements Serializable {

    private static final String LINK = "linked-endpoint";

    /**
     * 
     */
    private static final long serialVersionUID = 1L;

    /**
     * List of interfaces (null or only one interface)
     */
    protected QName[] interfaces;

    /**
     * Real endpoint name
     */
    protected String toEndpointName;

    /**
     * Real service name
     */
    protected QName toServiceName;

    /**
     * 
     * @param interfaceName
     * @param toServiceName
     *            real servicename
     * @param toEndpointName
     *            real endpoint name
     */
    public LinkedEndpoint(QName interfaceName, QName toServiceName,
        String toEndpointName, EndpointService endpointService) {
        super(null, null, LINK, LINK, EndpointType.LINKED, endpointService);
        this.interfaces = new QName[] {interfaceName};
        this.toServiceName = toServiceName;
        this.toEndpointName = toEndpointName;
    }

    /**
     * 
     * @param serviceName
     * @param endpointName
     * @param toServiceName
     *            real servicename
     * @param toEndpointName
     *            real endpoint name
     */
    public LinkedEndpoint(QName serviceName, String endpointName,
        QName toServiceName, String toEndpointName,
        EndpointService endpointService) {
        super(serviceName, endpointName, LINK, LINK, EndpointType.LINKED,
            endpointService);
        this.toServiceName = toServiceName;
        this.toEndpointName = toEndpointName;
    }

    public QName[] getInterfaces() {
        return interfaces;
    }

    public String getToEndpointName() {
        return toEndpointName;
    }

    public QName getToServiceName() {
        return toServiceName;
    }

    public void setToEndpointName(String toEndpointName) {
        this.toEndpointName = toEndpointName;
    }

    public void setToServiceName(QName toServiceName) {
        this.toServiceName = toServiceName;
    }
}
