package org.objectweb.petals.kernel.configuration;

import org.objectweb.fractal.api.NoSuchInterfaceException;
import org.objectweb.fractal.api.control.IllegalLifeCycleException;
import org.objectweb.fractal.fraclet.annotation.annotations.FractalComponent;
import org.objectweb.fractal.fraclet.annotation.annotations.Interface;
import org.objectweb.fractal.fraclet.annotation.annotations.LifeCycle;
import org.objectweb.fractal.fraclet.annotation.annotations.Monolog;
import org.objectweb.fractal.fraclet.annotation.annotations.Provides;
import org.objectweb.fractal.fraclet.annotation.annotations.type.LifeCycleType;
import org.objectweb.petals.util.LoggingUtil;
import org.objectweb.petals.util.SystemUtil;
import org.objectweb.util.monolog.api.Logger;

/**
 * Configuration as Fractal Component
 * 
 * @author Christophe DENEUX (Capgemini Sud)
 */
@FractalComponent
@Provides(interfaces = @Interface(name = "service", signature = LocalConfiguration.class))
public class LocalConfigurationImpl implements LocalConfiguration {

	/**
     * Fractal Logger
     */
    @Monolog(name = "logger")
    protected Logger logger;

    /**
     * Logger Wrapper
     */
    protected LoggingUtil log;

    /**
     * Init variables with Fractal attributes value and create/bind the topic
     * for this container.
     * 
     * @throws NoSuchInterfaceException
     * 
     * @throws Exception
     */
    @LifeCycle(on = LifeCycleType.START)
    public void start() throws IllegalLifeCycleException,
            NoSuchInterfaceException {
        this.log = new LoggingUtil(logger);
        this.log.start();
        
        this.log.end();
    }

    /**
     * Method called when the fractal component is stopped
     * 
     * @throws Exception
     */
    @LifeCycle(on = LifeCycleType.STOP)
    public void stop() throws Exception {
        log.start();
        log.end();
    }
    
    
    public String getValue(final String parameterName) {
		return SystemUtil.getAndUpdateValue(parameterName);
	}

	public String getValue(final String parameterName, final String defaultValue) {
		final String value = SystemUtil.getAndUpdateValue(parameterName);
		return (value == null || value.length() == 0) ? defaultValue : value;
	}

}
